@@ -50,6 +50,7 @@ import tempfile
 import tokenize
 import unittest
 import warnings
+import weakref
 from importlib import reload
 from importlib.abc import Loader, MetaPathFinder
 from importlib.machinery import ModuleSpec
@@ -57,24 +58,15 @@ from importlib.util import spec_from_file_location, module_from_spec
 from types import ModuleType, TracebackType, FunctionType
 from typing import (
     Any,
-    Callable,
-    Dict,
-    List,
-    Set,
-    Tuple,
     Optional,
-    Union,
-    Type,
-    Iterator,
     cast,
-    ItemsView,
-    Sequence,
 )
+
+from collections.abc import Callable, Iterator, ItemsView, Sequence
 from unittest import TestSuite
 
 from pyfakefs import fake_filesystem, fake_io, fake_os, fake_open, fake_path, fake_file
 from pyfakefs import fake_filesystem_shutil
-from pyfakefs import fake_legacy_modules
 from pyfakefs import fake_pathlib
 from pyfakefs import mox3_stubout
 from pyfakefs.fake_filesystem import (
@@ -86,7 +78,6 @@ from pyfakefs.fake_filesystem import (
 )
 from pyfakefs.fake_os import use_original_os
 from pyfakefs.helpers import IS_PYPY, IS_WIN
-from pyfakefs.legacy_packages import pathlib2, scandir
 from pyfakefs.mox3_stubout import StubOutForTesting
 
 OS_MODULE = "nt" if sys.platform == "win32" else "posix"
@@ -194,11 +185,11 @@ class LineCachePatcher:
 
 
 def patchfs(
-    _func: Optional[Callable] = None,
+    _func: Callable | None = None,
     *,
-    additional_skip_names: Optional[List[Union[str, ModuleType]]] = None,
-    modules_to_reload: Optional[List[ModuleType]] = None,
-    modules_to_patch: Optional[Dict[str, ModuleType]] = None,
+    additional_skip_names: list[str | ModuleType] | None = None,
+    modules_to_reload: list[ModuleType] | None = None,
+    modules_to_patch: dict[str, ModuleType] | None = None,
     allow_root_user: bool = True,
     use_known_patches: bool = True,
     patch_open_code: PatchMode = PatchMode.OFF,
@@ -261,9 +252,9 @@ def load_doctests(
     tests: TestSuite,
     ignore: Any,
     module: ModuleType,
-    additional_skip_names: Optional[List[Union[str, ModuleType]]] = None,
-    modules_to_reload: Optional[List[ModuleType]] = None,
-    modules_to_patch: Optional[Dict[str, ModuleType]] = None,
+    additional_skip_names: list[str | ModuleType] | None = None,
+    modules_to_reload: list[ModuleType] | None = None,
+    modules_to_patch: dict[str, ModuleType] | None = None,
     allow_root_user: bool = True,
     use_known_patches: bool = True,
     patch_open_code: PatchMode = PatchMode.OFF,
@@ -344,9 +335,9 @@ class TestCaseMixin:
                     methodName=methodName, modules_to_reload=[sut])
     """
 
-    additional_skip_names: Optional[List[Union[str, ModuleType]]] = None
-    modules_to_reload: Optional[List[ModuleType]] = None
-    modules_to_patch: Optional[Dict[str, ModuleType]] = None
+    additional_skip_names: list[str | ModuleType] | None = None
+    modules_to_reload: list[ModuleType] | None = None
+    modules_to_patch: dict[str, ModuleType] | None = None
 
     @property
     def patcher(self):
@@ -360,9 +351,9 @@ class TestCaseMixin:
 
     def setUpPyfakefs(
         self,
-        additional_skip_names: Optional[List[Union[str, ModuleType]]] = None,
-        modules_to_reload: Optional[List[ModuleType]] = None,
-        modules_to_patch: Optional[Dict[str, ModuleType]] = None,
+        additional_skip_names: list[str | ModuleType] | None = None,
+        modules_to_reload: list[ModuleType] | None = None,
+        modules_to_patch: dict[str, ModuleType] | None = None,
         allow_root_user: bool = True,
         use_known_patches: bool = True,
         patch_open_code: PatchMode = PatchMode.OFF,
@@ -409,9 +400,9 @@ class TestCaseMixin:
     @classmethod
     def setUpClassPyfakefs(
         cls,
-        additional_skip_names: Optional[List[Union[str, ModuleType]]] = None,
-        modules_to_reload: Optional[List[ModuleType]] = None,
-        modules_to_patch: Optional[Dict[str, ModuleType]] = None,
+        additional_skip_names: list[str | ModuleType] | None = None,
+        modules_to_reload: list[ModuleType] | None = None,
+        modules_to_patch: dict[str, ModuleType] | None = None,
         allow_root_user: bool = True,
         use_known_patches: bool = True,
         patch_open_code: PatchMode = PatchMode.OFF,
@@ -426,16 +417,9 @@ class TestCaseMixin:
         Note that using both :py:func:`setUpClassPyfakefs` and
         :py:func:`setUpPyfakefs` in the same class will not work correctly.
 
-        .. note:: This method is only available from Python 3.8 onwards.
         .. note:: If using `pytest` as testrunner, you need at least pytest 6.2
             for this method to work.
         """
-        if sys.version_info < (3, 8):
-            raise NotImplementedError(
-                "setUpClassPyfakefs is only available in "
-                "Python versions starting from 3.8"
-            )
-
         # if the class has already a patcher setup, we use this one
         if Patcher.PATCHER is not None:
             return
@@ -499,9 +483,9 @@ class TestCase(unittest.TestCase, TestCaseMixin):
     def __init__(
         self,
         methodName: str = "runTest",
-        additional_skip_names: Optional[List[Union[str, ModuleType]]] = None,
-        modules_to_reload: Optional[List[ModuleType]] = None,
-        modules_to_patch: Optional[Dict[str, ModuleType]] = None,
+        additional_skip_names: list[str | ModuleType] | None = None,
+        modules_to_reload: list[ModuleType] | None = None,
+        modules_to_patch: dict[str, ModuleType] | None = None,
     ):
         """Creates the test class instance and the patcher used to stub out
         file system related modules.
@@ -587,21 +571,21 @@ class Patcher:
 
     # caches all modules that do not have file system modules or function
     # to speed up _find_modules
-    CACHED_MODULES: Set[ModuleType] = set()
-    FS_MODULES: Dict[str, Set[Tuple[ModuleType, str]]] = {}
-    FS_FUNCTIONS: Dict[Tuple[str, str, str], Set[ModuleType]] = {}
-    FS_DEFARGS: List[Tuple[FunctionType, int, Callable[..., Any]]] = []
-    SKIPPED_FS_MODULES: Dict[str, Set[Tuple[ModuleType, str]]] = {}
+    CACHED_MODULES: set[ModuleType] = set()
+    FS_MODULES: dict[str, set[tuple[ModuleType, str]]] = {}
+    FS_FUNCTIONS: dict[tuple[str, str, str], set[ModuleType]] = {}
+    FS_DEFARGS: list[tuple[FunctionType, int, Callable[..., Any]]] = []
+    SKIPPED_FS_MODULES: dict[str, set[tuple[ModuleType, str]]] = {}
 
     assert None in SKIPMODULES, "sys.modules contains 'None' values; must skip them."
 
     IS_WINDOWS = sys.platform in ("win32", "cygwin")
 
-    SKIPNAMES: Set[str] = set()
+    SKIPNAMES: set[str] = set()
 
     # hold values from last call - if changed, the cache has to be invalidated
-    PATCHED_MODULE_NAMES: Set[str] = set()
-    ADDITIONAL_SKIP_NAMES: Set[str] = set()
+    PATCHED_MODULE_NAMES: set[str] = set()
+    ADDITIONAL_SKIP_NAMES: set[str] = set()
     PATCH_DEFAULT_ARGS = False
     PATCHER: Optional["Patcher"] = None
     DOC_PATCHER: Optional["Patcher"] = None
@@ -619,9 +603,9 @@ class Patcher:
 
     def __init__(
         self,
-        additional_skip_names: Optional[List[Union[str, ModuleType]]] = None,
-        modules_to_reload: Optional[List[ModuleType]] = None,
-        modules_to_patch: Optional[Dict[str, ModuleType]] = None,
+        additional_skip_names: list[str | ModuleType] | None = None,
+        modules_to_reload: list[ModuleType] | None = None,
+        modules_to_patch: dict[str, ModuleType] | None = None,
         allow_root_user: bool = True,
         use_known_patches: bool = True,
         patch_open_code: PatchMode = PatchMode.OFF,
@@ -704,29 +688,29 @@ class Patcher:
             ]
             self.skip_names.update(skip_names)
 
-        self._fake_module_classes: Dict[str, Any] = {}
-        self._unfaked_module_classes: Dict[str, Any] = {}
-        self._class_modules: Dict[str, List[str]] = {}
+        self._fake_module_classes: dict[str, Any] = {}
+        self._unfaked_module_classes: dict[str, Any] = {}
+        self._class_modules: dict[str, list[str]] = {}
         self._init_fake_module_classes()
 
         # reload tempfile under posix to patch default argument
-        self.modules_to_reload: List[ModuleType] = []
+        self.modules_to_reload: list[ModuleType] = []
         if modules_to_reload is not None:
             self.modules_to_reload.extend(modules_to_reload)
         self.patch_default_args = patch_default_args
         self.use_cache = use_cache
         self.use_dynamic_patch = use_dynamic_patch
-        self.cleanup_handlers: Dict[str, Callable[[str], bool]] = {}
+        self.cleanup_handlers: dict[str, Callable[[str], bool]] = {}
 
         # Attributes set by _refresh()
-        self._stubs: Optional[StubOutForTesting] = None
-        self.fs: Optional[FakeFilesystem] = None
-        self.fake_modules: Dict[str, Any] = {}
-        self.unfaked_modules: Dict[str, Any] = {}
+        self._stubs: StubOutForTesting | None = None
+        self.fs: FakeFilesystem | None = None
+        self.fake_modules: dict[str, Any] = {}
+        self.unfaked_modules: dict[str, Any] = {}
 
         # _isStale is set by tearDown(), reset by _refresh()
         self._isStale = True
-        self._dyn_patcher: Optional[DynamicPatcher] = None
+        self._dyn_patcher: DynamicPatcher | None = None
         self._patching = False
         self._paused = False
         self.has_copy_file_range = False
@@ -766,7 +750,7 @@ class Patcher:
 
         if clear_cache:
             self.clear_cache()
-        self._fake_module_functions: Dict[str, Dict] = {}
+        self._fake_module_functions: dict[str, dict] = {}
         self._init_fake_module_functions()
 
     @classmethod
@@ -842,14 +826,6 @@ class Patcher:
             self._unfaked_module_classes["pathlib._local"] = (
                 fake_pathlib.RealPathlibModule
             )
-        if pathlib2:
-            self._fake_module_classes["pathlib2"] = (
-                fake_legacy_modules.FakePathlib2Module
-            )
-            self._class_modules["Path"].append("pathlib2")
-            self._unfaked_module_classes["pathlib2"] = fake_pathlib.RealPathlibModule
-        if scandir:
-            self._fake_module_classes["scandir"] = fake_legacy_modules.FakeScanDirModule
         self._fake_module_classes["Path"] = fake_path_module
         self._unfaked_module_classes["Path"] = fake_pathlib.RealPathlibPathModule
 
@@ -893,14 +869,14 @@ class Patcher:
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         self.tearDown()
 
     def _is_fs_module(
-        self, mod: ModuleType, name: str, module_names: List[str]
+        self, mod: ModuleType, name: str, module_names: list[str]
     ) -> bool:
         try:
             return (
@@ -929,7 +905,7 @@ class Patcher:
 
     def _def_values(
         self, item: FunctionType
-    ) -> Iterator[Tuple[FunctionType, int, Any]]:
+    ) -> Iterator[tuple[FunctionType, int, Any]]:
         """Find default arguments that are file-system functions to be
         patched in top-level functions and members of top-level classes."""
         # check for module-level functions
@@ -1037,7 +1013,12 @@ class Patcher:
         for name in self._fake_module_classes:
             self.fake_modules[name] = self._fake_module_classes[name](self.fs)
             if hasattr(self.fake_modules[name], "skip_names"):
-                self.fake_modules[name].skip_names = self.skip_names
+                self.fake_modules[name].skip_names = self.skip_names | {
+                    # also skip non-build-in skipped modules
+                    m.__name__
+                    for m in self.SKIPMODULES
+                    if m and "." in m.__name__
+                }
         self.fake_modules[PATH_MODULE] = self.fake_modules["os"].path
         for name in self._unfaked_module_classes:
             self.unfaked_modules[name] = self._unfaked_module_classes[name]()
@@ -1101,6 +1082,8 @@ class Patcher:
             globber.lstat = staticmethod(os.lstat)
             if sys.version_info < (3, 14):
                 globber.scandir = staticmethod(os.scandir)
+        if sys.version_info >= (3, 14):
+            globber.lexists = staticmethod(os.path.lexists)
 
     def patch_functions(self) -> None:
         assert self._stubs is not None
@@ -1155,7 +1138,7 @@ class Patcher:
                     new_defaults.append(d)
             fct.__defaults__ = tuple(new_defaults)
 
-    def replace_globs(self, globs_: Dict[str, Any]) -> Dict[str, Any]:
+    def replace_globs(self, globs_: dict[str, Any]) -> dict[str, Any]:
         globs = globs_.copy()
         if self._isStale:
             self._refresh()
@@ -1204,7 +1187,7 @@ class Patcher:
     def unset_defaults(self) -> None:
         for fct, idx, ft in self.FS_DEFARGS:
             new_defaults = []
-            for i, d in enumerate(cast(Tuple, fct.__defaults__)):
+            for i, d in enumerate(cast(tuple, fct.__defaults__)):
                 if i == idx:
                     new_defaults.append(ft)
                 else:
@@ -1236,7 +1219,7 @@ class Pause:
     going out of its scope.
     """
 
-    def __init__(self, caller: Union[Patcher, TestCaseMixin, FakeFilesystem]):
+    def __init__(self, caller: Patcher | TestCaseMixin | FakeFilesystem):
         """Initializes the context manager with the fake filesystem.
 
         Args:
@@ -1271,10 +1254,10 @@ class DynamicPatcher(MetaPathFinder, Loader):
     """
 
     def __init__(self, patcher: Patcher) -> None:
-        self._patcher = patcher
+        self._patcher = weakref.ref(patcher)
         self.sysmodules = {}
-        self.modules = self._patcher.fake_modules
-        self._loaded_module_names: Set[str] = set()
+        self.modules = self.patcher.fake_modules
+        self._loaded_module_names: set[str] = set()
         self.cleanup_handlers = patcher.cleanup_handlers
 
         # remove all modules that have to be patched from `sys.modules`,
@@ -1290,11 +1273,11 @@ class DynamicPatcher(MetaPathFinder, Loader):
     def cleanup(self) -> None:
         for module_name in self.sysmodules:
             sys.modules[module_name] = self.sysmodules[module_name]
-        for module in self._patcher.modules_to_reload:
+        for module in self.patcher.modules_to_reload:
             if module.__name__ in sys.modules:
                 reload(module)
         reloaded_module_names = [
-            module.__name__ for module in self._patcher.modules_to_reload
+            module.__name__ for module in self.patcher.modules_to_reload
         ]
         # Delete all modules loaded during the test, ensuring that
         # they are reloaded after the test.
@@ -1317,7 +1300,7 @@ class DynamicPatcher(MetaPathFinder, Loader):
         """Checks if the module with the given name is a module existing in the fake
         filesystem and returns its path in this case.
         """
-        fs = self._patcher.fs
+        fs = self.patcher.fs
         # we assume that the module name is the absolute module path
         if fs is not None:
             base_path = name.replace(".", fs.path_separator)
@@ -1331,16 +1314,22 @@ class DynamicPatcher(MetaPathFinder, Loader):
                     return fs.absnormpath(init_path)
         return ""
 
+    @property
+    def patcher(self):
+        p = self._patcher()
+        assert p is not None
+        return p
+
     def find_spec(
         self,
         fullname: str,
-        path: Optional[Sequence[Union[bytes, str]]],
-        target: Optional[ModuleType] = None,
-    ) -> Optional[ModuleSpec]:
+        path: Sequence[bytes | str] | None,
+        target: ModuleType | None = None,
+    ) -> ModuleSpec | None:
         """Module finder."""
         if self.needs_patch(fullname):
             return ModuleSpec(fullname, self)
-        if self._patcher.patch_open_code != PatchMode.OFF:
+        if self.patcher.patch_open_code != PatchMode.OFF:
             # handle modules created in the fake filesystem
             module_path = self.fake_module_path(fullname)
             if module_path:
